import sequelize from 'sequelize';
import IDbConfig from './interfaces/db-config-interface';
import PaginationLinks from './pagination-links';
import IPaginationOptions from './interfaces/pagination-options-interface';
/**
 * ServiceModel a helper for sequelize
 */
export default class ServiceModel {
    /**
     * Gets the sequelize constructor
     * @return {Sequelize}
     */
    static getSequelize(): {};
    /**
     * Validates the DB config
     *
     * @param {IDbConfig} config
     * The config for creating a new DB instance
     *
     * @return {boolean}
     * True/false based on config validity
     */
    static isValidDbConfig(config: IDbConfig): boolean;
    /**
     * Function to create the DB connection
     *
     * @param  {IDbConfig} config
     * variables including db params
     *
     * @return {sequelize.Sequelize}
     * sequelize db object
     */
    static createDb(config: IDbConfig): sequelize.Sequelize;
    /**
     * Returns an object containing the appropriate pagination links
     * based on the number of results, pageNumber currently accessing and pageSize.
     *
     * @param {object} paginationOptions
     * Pagination options object
     *
     * @param {number} paginationOptions.count
     * The number of results produced from the query
     *
     * @param {number} paginationOptions.offset
     * The results offset currently being accessed
     *
     * @param {number} paginationOptions.limit
     * The size of one page
     *
     * @param {string} paginationOptions.baseLink
     * Link to the endpoint that needs pagination
     * Ex: https://services.packpub.com/offers
     *
     * @return {PaginationLinks}
     * Containing the next and previous links
     */
    static generatePaginationLinks(paginationOptions: IPaginationOptions): PaginationLinks;
    db: sequelize.Sequelize;
    /**
     * @constructor
     * Constructor for services class. Creates a sequelize object on initialisation
     * @param  {IDbConfig} config
     * The database config object
     */
    constructor(config: IDbConfig);
    /**
     * Returns instance of the db object
     *
     * @return {object}
     * sequelize db instance
     */
    getDb(): object;
    /**
     * Closes the db connection.
     */
    closeDb(): void;
    /**
     * Checks the db connection
     *
     * @return {promise}
     * sequelize promise object
     */
    checkDbConnectivity(): Promise<void | Error>;
}
