"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const joi_1 = __importDefault(require("@hapi/joi"));
const sequelize_1 = __importDefault(require("sequelize"));
const url_1 = require("url");
const pagination_links_1 = __importDefault(require("./pagination-links"));
/**
 * ServiceModel a helper for sequelize
 */
class ServiceModel {
    /**
     * Gets the sequelize constructor
     * @return {Sequelize}
     */
    static getSequelize() {
        return sequelize_1.default;
    }
    /**
     * Validates the DB config
     *
     * @param {IDbConfig} config
     * The config for creating a new DB instance
     *
     * @return {boolean}
     * True/false based on config validity
     */
    static isValidDbConfig(config) {
        const joiConfigSchema = {
            dbHost: joi_1.default.string().required(),
            dbName: joi_1.default.string().required(),
            dbPass: joi_1.default.string().required(),
            dbUser: joi_1.default.string().required(),
            debug: joi_1.default.boolean(),
            userId: joi_1.default.string().guid(),
        };
        const joiValidationResult = joi_1.default.validate(config, joiConfigSchema);
        return !joiValidationResult.error;
    }
    /**
     * Function to create the DB connection
     *
     * @param  {IDbConfig} config
     * variables including db params
     *
     * @return {sequelize.Sequelize}
     * sequelize db object
     */
    static createDb(config) {
        return new sequelize_1.default.Sequelize(config.dbName, config.dbUser, config.dbPass, {
            dialect: 'postgres',
            host: config.dbHost,
            logging: config.debug || false,
        });
    }
    /**
     * Returns an object containing the appropriate pagination links
     * based on the number of results, pageNumber currently accessing and pageSize.
     *
     * @param {object} paginationOptions
     * Pagination options object
     *
     * @param {number} paginationOptions.count
     * The number of results produced from the query
     *
     * @param {number} paginationOptions.offset
     * The results offset currently being accessed
     *
     * @param {number} paginationOptions.limit
     * The size of one page
     *
     * @param {string} paginationOptions.baseLink
     * Link to the endpoint that needs pagination
     * Ex: https://services.packpub.com/offers
     *
     * @return {PaginationLinks}
     * Containing the next and previous links
     */
    static generatePaginationLinks(paginationOptions) {
        const paginationOptionsJoiSchema = {
            baseLink: joi_1.default
                .string()
                .uri({
                scheme: 'https',
            })
                .required(),
            count: joi_1.default
                .number()
                .options({ convert: false })
                .integer()
                .min(0)
                .required(),
            limit: joi_1.default
                .number()
                .options({ convert: false })
                .min(1)
                .required(),
            offset: joi_1.default
                .number()
                .options({ convert: false })
                .min(0),
        };
        const validatedPaginationOptions = joi_1.default.validate(paginationOptions, paginationOptionsJoiSchema);
        if (!paginationOptions || validatedPaginationOptions.error) {
            throw new Error('Please provide valid pagination options.');
        }
        const { count, offset = 0, limit } = paginationOptions;
        const baseUrl = new url_1.URL(paginationOptions.baseLink);
        const hasResults = count > 0;
        const hasPrev = hasResults && offset >= 1;
        const hasNext = offset < count - limit;
        const links = new pagination_links_1.default();
        if (hasPrev) {
            const newOffset = offset < limit ? 0 : offset - limit;
            baseUrl.searchParams.set('offset', newOffset.toString());
            baseUrl.searchParams.set('limit', limit.toString());
            links.prev = baseUrl.toString();
        }
        if (hasNext) {
            const newOffset = offset + limit;
            baseUrl.searchParams.set('offset', newOffset.toString());
            baseUrl.searchParams.set('limit', limit.toString());
            links.next = baseUrl.toString();
        }
        return links;
    }
    /**
     * @constructor
     * Constructor for services class. Creates a sequelize object on initialisation
     * @param  {IDbConfig} config
     * The database config object
     */
    constructor(config) {
        if (!ServiceModel.isValidDbConfig(config)) {
            throw new Error('Invalid DB credentials');
        }
        this.db = ServiceModel.createDb(config);
    }
    /**
     * Returns instance of the db object
     *
     * @return {object}
     * sequelize db instance
     */
    getDb() {
        return this.db;
    }
    /**
     * Closes the db connection.
     */
    closeDb() {
        this.db.close();
    }
    /**
     * Checks the db connection
     *
     * @return {promise}
     * sequelize promise object
     */
    checkDbConnectivity() {
        return __awaiter(this, void 0, void 0, function* () {
            return this.db
                .authenticate()
                .catch(() => Promise.reject(new Error('Unable to connect to the database')));
        });
    }
}
exports.default = ServiceModel;
//# sourceMappingURL=service-model.js.map